@@ -1,4 +1,4 @@
-use crate::{CmdResult, FunResult};
+use crate::{process, CmdResult, FunResult};
 use log::{error, info, warn};
 use os_pipe::PipeReader;
 use std::io::{BufRead, BufReader, Error, ErrorKind, Read, Result};
@@ -129,15 +129,6 @@ pub(crate) enum CmdChild {
 
 impl CmdChild {
     fn wait(self, is_last: bool) -> CmdResult {
-        let pipefail = std::env::var("CMD_LIB_PIPEFAIL") != Ok("0".into());
-        let check_result = |result| {
-            if let Err(e) = result {
-                if is_last || pipefail {
-                    return Err(e);
-                }
-            }
-            Ok(())
-        };
         match self {
             CmdChild::Proc {
                 mut child,
@@ -151,11 +142,15 @@ impl CmdChild {
                     Self::wait_logging_thread(&cmd, polling_stderr);
                 }
                 Self::print_stdout_output(&mut child.stdout);
-                if !ignore_error && !status.success() && (is_last || pipefail) {
-                    return Err(Self::status_to_io_error(
-                        status,
-                        &format!("{} exited with error", cmd),
-                    ));
+                if !status.success() {
+                    if !ignore_error && (is_last || process::pipefail_enabled()) {
+                        return Err(Self::status_to_io_error(
+                            status,
+                            &format!("{} exited with error", cmd),
+                        ));
+                    } else if process::debug_enabled() {
+                        warn!("{} exited with error: {}", cmd, status);
+                    }
                 }
             }
             CmdChild::ThreadFn {
@@ -170,20 +165,21 @@ impl CmdChild {
                 if let Some(polling_stderr) = polling_stderr {
                     Self::wait_logging_thread(&cmd, polling_stderr);
                 }
-                if ignore_error {
-                    return Ok(());
-                }
                 match status {
                     Err(e) => {
-                        if is_last || pipefail {
-                            return Err(Error::new(
-                                ErrorKind::Other,
-                                format!("{} thread exited with error: {:?}", cmd, e),
-                            ));
-                        }
+                        return Err(Error::new(
+                            ErrorKind::Other,
+                            format!("{} thread joined with error: {:?}", cmd, e),
+                        ));
                     }
                     Ok(result) => {
-                        check_result(result)?;
+                        if let Err(e) = result {
+                            if !ignore_error && (is_last || process::pipefail_enabled()) {
+                                return Err(e);
+                            } else if process::debug_enabled() {
+                                warn!("{} exited with error: {:?}", cmd, e);
+                            }
+                        }
                     }
                 }
             }
@@ -221,20 +217,24 @@ impl CmdChild {
                 if let Some(polling_stderr) = polling_stderr {
                     Self::wait_logging_thread(&cmd, polling_stderr);
                 }
-                if !ignore_error && !output.status.success() {
-                    return Err(Self::status_to_io_error(
-                        output.status,
-                        &format!("{} exited with error", cmd),
-                    ));
-                } else {
-                    Ok(output.stdout)
+                if !output.status.success() {
+                    if !ignore_error {
+                        return Err(Self::status_to_io_error(
+                            output.status,
+                            &format!("{} exited with error", cmd),
+                        ));
+                    } else if process::debug_enabled() {
+                        warn!("{} exited with error: {}", cmd, output.status);
+                    }
                 }
+                Ok(output.stdout)
             }
             CmdChild::ThreadFn {
                 cmd,
                 stdout,
                 stderr,
                 child,
+                ignore_error,
                 ..
             } => {
                 let polling_stderr = stderr.map(CmdChild::log_stderr_output);
@@ -246,10 +246,27 @@ impl CmdChild {
                 } else {
                     vec![]
                 };
-                child.join().unwrap()?;
+                let status = child.join();
                 if let Some(polling_stderr) = polling_stderr {
                     Self::wait_logging_thread(&cmd, polling_stderr);
                 }
+                match status {
+                    Err(e) => {
+                        return Err(Error::new(
+                            ErrorKind::Other,
+                            format!("{} thread joined with error: {:?}", cmd, e),
+                        ));
+                    }
+                    Ok(result) => {
+                        if let Err(e) = result {
+                            if !ignore_error {
+                                return Err(e);
+                            } else if process::debug_enabled() {
+                                warn!("{} exited with error: {:?}", cmd, e);
+                            }
+                        }
+                    }
+                }
                 Ok(buf)
             }
             CmdChild::SyncFn {
